#-----------------------------------------------------------------------------
# Project  Tutoriels - Conception de circuits intégrés numériques
#-----------------------------------------------------------------------------
# File     pwr.tcl
# Authors  Mickael Fiorentino <mickael.fiorentino@polymtl.ca>
# Lab      GRM - Polytechnique Montréal
# Date     <2019-10-02 Wed>
#-----------------------------------------------------------------------------
# Brief    Estimation du puissance du compteur BCD avec Voltus
# Usage    source setup.csh && cd implementation/pnr
#          voltus -files ../scripts/pwr.tcl
#-----------------------------------------------------------------------------
package require Tcl 8.5

if { ![info exist ::env(PROJECT_HOME)] } {
    error "ERREUR: Configurer l'environnement (source setup.csh) avant d'utiliser ce script"
}

#-----------------------------------------------------------------------------
# CONFIG
#-----------------------------------------------------------------------------
set DO_INIT  1
set DO_STAT  1
set DO_DYN   0

set DESIGN     compteur
set OA         compteur_oa
set DUT        compteur_tb/dut
set PERIOD     1250ps
set RESOLUTION 100ps
set VCD_START  0ps
set VCD_END    54375ps

#-----------------------------------------------------------------------------
# INIT
#-----------------------------------------------------------------------------
if { $DO_INIT } {

    cd $::env(PNR_DIR)

    # Unités par défaut
    set_time_unit -picoseconds

    # Importation du layout
    read_design -cellview "$OA $DESIGN layout" -physical_data

    # Mise à jour du timing à partir du mmmc
    set_analysis_view -update_timing

    # Configuration
    set_pg_library_mode -extraction_tech_file $::env(BE_QRC_LIB)/gpdk045.tch -celltype techonly \
                        -power_pins {VDD 1.1} -ground_pins VSS                                  \
                        -decap_cells DECAP* -filler_cells FILL*                                 \

    generate_pg_library -output $::env(PNR_REP_DIR)/power
    cd ../
}

#-----------------------------------------------------------------------------
# ANALYSE STATIQUE
#-----------------------------------------------------------------------------
if { $DO_STAT } {

    set rep ${DESIGN}.stat

    # Configuration statique
    set_power_analysis_mode -reset
    set_power_analysis_mode -method static               \
                            -corner max                  \
                            -create_binary_db false      \
                            -write_static_currents false

    # Activité moyenne
    set_default_switching_activity -reset
    set_default_switching_activity -input_activity 0.2 -period $PERIOD
}

#-----------------------------------------------------------------------------
# ANALYSE DYNAMIQUE
#-----------------------------------------------------------------------------
if { $DO_DYN } {

    set rep ${DESIGN}.dyn

    # Configuration dynamique
    set_power_analysis_mode -reset
    set_power_analysis_mode -method dynamic_vectorbased                               \
                            -enable_rtl_vectorbased_dynamic_analysis true             \
                            -power_grid_library $::env(PNR_REP_DIR)/power/techonly.cl \
                            -corner max                                               \
                            -report_stat true                                         \
                            -create_binary_db true                                    \
                            -write_static_currents true                               \
                            -report_missing_flop_outputs false

    # Activité issue du fichier VCD
    set_default_switching_activity -reset
    read_activity_file -reset
    read_activity_file -format VCD $::env(SIM_DIR)/pnr/${DESIGN}.vcd \
                       -scope  $DUT -start $VCD_START -end $VCD_END

    set_dynamic_power_simulation -resolution $RESOLUTION
}

# Report power
report_power -format detailed -report_prefix $rep -output $::env(PNR_REP_DIR)/power
file copy -force $::env(PNR_REP_DIR)/power/${rep}.rpt $::env(PNR_REP_DIR)/${rep}.rpt
